import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The protagonist of the game, controlled by the user.
 */
public class BobRoss extends Actor implements Movable{
    
    private int speed;
    private int jumpSpeed;
    private int maxJump;
    private int vSpeed;
    private int accel;
    private int frame;
    private boolean left;
    private boolean isStill;
    private long paintCooldown;
    private boolean dead;

    /**
     * The sole constructor for a BobRoss object. Sets the values of each movement 
     * variable to a specific value.
     */
    public BobRoss(){
        speed = 25;
        jumpSpeed = 0;
        vSpeed = 0;
        maxJump = 100;
        accel = 10;
        frame = 0;
        isStill = true;
        paintCooldown = 0;
        dead = false;
    }
    
    /**
     * Checks for user input and Bob acts accordingly. This also checks if Bob 
     * is falling, has died, it also checks for palettes.
     */
    public void act(){
        isStill = true;
        checkInput();
        fall();
        takePalette();
        if(isStill && !left) setImage("BRStandR_0.png");
        else if (isStill && left) setImage("BRStandL_0.png");
        if(happyAccident() || dead) {
            getWorld().addObject(new Image("YouDied-Text.png"), 750, 200);
            if(dead) {
                try {
                    Thread.sleep(2000);
                } catch(InterruptedException | IllegalMonitorStateException ex) {}
                Greenfoot.setWorld(new Level(((Level)getWorld()).getLevelNum()));
            }
            dead = true;
        }
    }    
    
    /**
     * Checks for specific key presses and acts accordingly.
     */
    public void checkInput() {
        if(Greenfoot.isKeyDown("Left") && !Greenfoot.isKeyDown("Right")) {
            left = true;
            isStill = false;
            walkL();
        }
        if(Greenfoot.isKeyDown("Right") && !Greenfoot.isKeyDown("Left")) {
            left = false;
            isStill = false;
            walkR();
        }
        if(Greenfoot.isKeyDown("Up")) {
            isStill = false;
            jump();
        }
        if(Greenfoot.isKeyDown("Space")) {
            isStill = false;
            attack();
        }
        if(Greenfoot.isKeyDown("Left") && !Greenfoot.isKeyDown("Right") && Greenfoot.isKeyDown("Space")) {
            left = true;
            isStill = false;
            attackWalkL();
        }
        if(Greenfoot.isKeyDown("Right") && !Greenfoot.isKeyDown("Left") && Greenfoot.isKeyDown("Space")) {
            left = false;
            isStill = false;
            attackWalkR();
        }
    }
    
    
    /**
     * Moves the character left and animates walking.
     */
    public void walkL() {
        if(frame == 0)  setImage("BRWalkL_0.png");
        if(frame == 2) setImage("BRWalkL_1.png");
        if(frame == 4) setImage("BRWalkL_2.png");
        if(frame == 6)setImage("BRWalkL_3.png");
        if(frame == 8) setImage("BRWalkL_4.png");
        if(frame == 10) {
        setImage("BRWalkL_5.png");
            frame = 0;
            setLocation(getX() - speed, getY());
            return;
        }
        setLocation(getX() - speed, getY());
        frame++;
    }
    
    /**
     * Moves the character right and animates walking.
     */
    public void walkR() {
        if(frame == 0) setImage("BRWalkR_0.png");
        if(frame == 2) setImage("BRWalkR_1.png");
        if(frame == 4) setImage("BRWalkR_2.png");
        if(frame == 6) setImage("BRWalkR_3.png");
        if(frame == 8) setImage("BRWalkR_4.png");
        if(frame == 10) {
            setImage("BRWalkR_5.png");
            frame = 0;
            setLocation(getX() + speed, getY());
            return;
        }
        setLocation(getX() + speed, getY());
        frame++;
    }
    
    /**
     * Sets the image to attack mode and launches projectile with fire().
     */
    public void attack() {
        int offset = 0;
        if(left){
            setImage("BRAttackL.png");
            offset = -90;
        }
        else {
            setImage("BRAttackR.png");
            offset = 90;
        }
        fire(offset);
    }
    
    /**
     *  Sets the image to attack mode walking right and launches projectile with fire().
     */
    public void attackWalkR() {
        if(frame == 0) setImage("BRAttackWalkR_0.png");
        if(frame == 2) setImage("BRAttackWalkR_1.png");
        if(frame == 4) setImage("BRAttackWalkR_2.png");
        if(frame == 6) setImage("BRAttackWalkR_3.png");
        if(frame == 8) setImage("BRAttackWalkR_4.png");
        if(frame == 10)setImage("BRAttackWalkR_5.png");
        fire(90);
    }
    
    /**
     * Sets the image to attack mode walking left and launches projectile with fire().
     */
    public void attackWalkL() {
        if(frame == 0)  setImage("BRAttackWalkL_0.png");
        if(frame == 2) setImage("BRAttackWalkL_1.png");
        if(frame == 4) setImage("BRAttackWalkL_2.png");
        if(frame == 6)setImage("BRAttackWalkL_3.png");
        if(frame == 8) setImage("BRAttackWalkL_4.png");
        if(frame == 10) setImage("BRAttackWalkL_5.png");
        fire(-90);
    }
    
    /**
     * If the cooldown has elapsed, fire a projectile from the character's location.
     */
    public void fire() {
        if (System.currentTimeMillis() - paintCooldown >= 1000) {
            getWorld().addObject(new PaintSplat(left), getX(), getY() + 22  );
            paintCooldown = System.currentTimeMillis();
        }
    }
    
    /**
     * If the cooldown has elapsed, fire a projectile at an offset from the character's location.
     */
    public void fire(int offset) {
        if (System.currentTimeMillis() - paintCooldown >= 1000) {
            getWorld().addObject(new PaintSplat(left), getX() + offset, getY() + 22  );
            paintCooldown = System.currentTimeMillis();
        }
    }
    
    /**
     * Makes the character jump, accelerating until maxJump is reached.
     */ 
    public void jump() {
        if(jumpSpeed <= 0) {
            jumpSpeed = maxJump;
            return;
        }
        if(onGround()) {
            jumpSpeed = maxJump;
        }
        if(left) {
            setImage("BRJumpL.png");
        } else {
            setImage("BRJumpR.png");
        }
        jumpSpeed -= accel;
        setLocation(getX(), getY() - jumpSpeed);
    }
    
    /**
     * Makes the character fall until the ground is reached.
     */
    public void fall() {
        if(onGround()){
            vSpeed = 0;
            return;
        }
        vSpeed += accel;
        setLocation(getX(), getY() + vSpeed);
        antiNoClip();
   }

   /**
    * Sets the character above ground if the fall() method caused the character to 
    * clip through the floor.
    */
   public void antiNoClip() {
       GreenfootImage img = getImage();
       setImage("HitBox.png");
       Actor clipTile = getOneIntersectingObject(Tiles.class);
       setImage(img);
       if(clipTile != null) {

            int newY = clipTile.getY() - (clipTile.getImage().getHeight() + getImage().getHeight())/2;
            setLocation(getX(), newY);
       }
   }

    /**
     * @return whether or not the character is standing within one pixel of the ground from
     * his foot.
     */
   public boolean onGround() {

        Actor belowL = getOneObjectAtOffset(-15, (getImage().getHeight()/2), Tiles.class);
        Actor belowR = getOneObjectAtOffset(15, (getImage().getHeight()/2), Tiles.class);
        Actor below = getOneObjectAtOffset(0, (getImage().getHeight()/2), Tiles.class);
        return (belowL != null || belowR != null || below != null);
    }

    /**
     * @return whether or not the charater has touched an EvilRabbit or fallen off the map.
     */
   public boolean happyAccident() {
        Actor enemy = getOneObjectAtOffset(15, 0, EvilRabbit.class);
        if(enemy != null) return true;
        enemy = getOneObjectAtOffset(-15, 0, EvilRabbit.class);
        if(enemy != null) return true;
        if(getY() > 394) return true;
        return false;
    }
    
    /**
     * Removes a palette the character is touching, if any.
     */
   public void takePalette() {
        Actor palette = getOneObjectAtOffset(15, 0, Palette.class);
        if(palette != null) removeTouching(Palette.class);
        palette = getOneObjectAtOffset(-15, 0, Palette.class);
        if(palette != null) removeTouching(Palette.class);
   }

    /**
     * @return the character's walking speed.
     */
    public int getSpeed() {
        return speed;
    }

    /**
     * @precondition must be in increments of 5.
     * @param the character's walking speed.
     */
    public void setSpeed(int speed) {
        this.speed = speed;
    }

    /**
     * @precondition must be in increments of 5.
     * @return the character's jumping speed.
     */
    public int getJumpSpeed() {
        return jumpSpeed;
    }

    /**
     * @precondition must be in increments of 5.
     * @param jumpSpeed the character's jumping speed.
     */
    public void setJumpSpeed(int jumpSpeed) {
        this.jumpSpeed = jumpSpeed;
    }

    /**
     * @precondition must be in increments of 5.
     * @return the character's maximum jumping speed.
     */
    public int getMaxJump() {
        return maxJump;
    }

    /**
     * @precondition must be in increments of 5.
     * @param maxJump the character's maximum jumping speed.
     */
    public void setMaxJump(int maxJump) {
        this.maxJump = maxJump;
    }

    /**
     * @precondition must be in increments of 5.
     * @return the character's falling speed;
     */
    public int getVSpeed() {
        return vSpeed;
    }

    /**
     * @precondition must be in increments of 5.
     * @param vSpeed the character's falling speed;
     */
    public void setVSpeed(int vSpeed) {
        this.vSpeed = vSpeed;
    }

    /**
     * @precondition must be in increments of 5.
     * @return the character's vertical acceleration.
     */
    public int getAccel() {
        return accel;
    }

    /**
     * @precondition must be in increments of 5.
     * @param accel the character's vertical acceleration.
     */
    public void setAccel(int accel) {
        this.accel = accel;
    }

    /**
     * @return the animation frame that the charcter is in.
     */
    public int getFrame() {
        return frame;
    }

    /**
     * @param frame the animation frame that the charcter is in.
     */
    public void setFrame(int frame) {
        this.frame = frame;
    }

    /**
     * @return whether or not the character is facing left.
     */
    public boolean isLeft() {
        return left;
    }

    /**
     * @param left whether or not the character is facing left.
     */
    public void setLeft(boolean left) {
        this.left = left;
    }

    /**
     * @return whether or not the character is standing still.
     */
    public boolean isStill() {
        return isStill;
    }

    /**
     * @param still whether or not the character is standing still.
     */
    public void setStill(boolean still) {
        isStill = still;
    }

    /**
     * @return whether or not the character is dead.
     */
    public boolean isDead() {
        return dead;
    }

    /**
     * @param dead whether or not the character is dead.
     */
    public void setDead(boolean dead) {
        this.dead = dead;
    }

    /**
     * @return a string representation of the object.
     */
    public String toString() {
        return "BobRoss{" +
                "speed = " + speed +
                ", jumpSpeed = " + jumpSpeed +
                ", maxJump = " + maxJump +
                ", vSpeed = " + vSpeed +
                ", accel = " + accel +
                ", frame = " + frame +
                ", left =  " + left +
                ", isStill = " + isStill +
                ", paintCooldown = " + paintCooldown +
                ", dead = " + dead +
                '}';
    }
}