import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A Rabbit that kills BobRoss if he comes into contact with it.
 * Turns into a GoodRabbit if it collides with a PaintSplat.
 */
public class EvilRabbit extends Rabbit {
    
    private int height, width;
    private static final String STAND_L = "ERStandL.png";
    private static final String STAND_R = "ERStandR.png";
    private static final String[] WALK_L = {"ERWalkL_0.png", "ERWalkL_1.png", "ERWalkL_2.png", "ERWalkL_3.png", "ERWalkL_4.png", "ERWalkL_5.png", "ERWalkL_6.png"};
    private static final String[] WALK_R = {"ERWalkR_0.png", "ERWalkR_1.png", "ERWalkR_2.png", "ERWalkR_3.png", "ERWalkR_4.png", "ERWalkR_5.png", "ERWalkR_6.png"};
    
    /**
     * Constructs an EvilRabbit with the default animation frames. 
     */
    public EvilRabbit() {
        super(STAND_L, STAND_R, WALK_L, WALK_R);
        height = getImage().getHeight();
        width = getImage().getWidth();
    }

    /**
     * Constructs an EvilRabbit with the default animation frames facing
     * a specified direction.
     *
     * @param isLeft whether or not the character is facing left.
     */
    public EvilRabbit(boolean isLeft) {
        super(isLeft, STAND_L, STAND_R, WALK_L, WALK_R);
        height = getImage().getHeight();
        width = getImage().getWidth();
    }

    /**
     * Make the EvilRabbit move in a random directions for a random amount of steps.
     * The EvilRabbit waits for two seconds after a set of steps is completed.
     */
    public void act() {
        super.act();
    }

    /**
     * @return the height of the image for the character.
     */
    public int getHeight() {
        return height;
    }

    /**
     * @param height the height of the image for the character.
     */
    public void setHeight(int height) {
        this.height = height;
    }

    /**
     * @return the width of the image for the character.
     */
    public int getWidth() {
        return width;
    }

    /**
     * @param width the width of the image for the character.
     */
    public void setWidth(int width) {
        this.width = width;
    }

    /**
     * @return a string representation of the object.
     */
    public String toString() {
        return "EvilRabbit{" +
                "height = " + height +
                ", width = " + width +
                '}';
    }
}