import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.ArrayList;
import java.util.List;

/**
 * Creates levels based on levelNum. This controls the main flow of the 
 * game from level to level.
 */
public class Level extends World {
    private boolean loaded;
    private boolean paletteAdded;
    private boolean victory;
    private static long musicStart; 
    private ArrayList<EvilRabbit> enemies;
    private int levelNum;
    private int paletteX;
    private int paletteY;

    /**
     * Constructor for objects of class Level.
     * 
     * @param levelNum the level that is to be loaded.
     */
    public Level(int levelNum) {    
        super(1500, 400, 1);
        loaded = false;
        paletteAdded = false;
        this.levelNum = levelNum;
        victory = false;
    }

    /**
     * Incites the initial setup of the level.
     * Then it checks if any EvilRabbits are present and adds the palette if there aren't.
     */
    public void act(){
        if(!loaded) {
            initialLoad();
            loaded = true;
        }
        if(victory) {
            if(Greenfoot.mouseClicked(this)) {
                Greenfoot.setWorld(new MainMenu());
            }
            return;
        }
        if(!isEnemy() && !paletteAdded){
            addObject(new Palette(), paletteX, paletteY);
            paletteAdded = true;
        }
        if(paletteAdded) {
            List<Palette> palettes = getObjects(Palette.class);
            if(palettes.size() == 0) Greenfoot.setWorld(new Level(getLevelNum() + 1));
        }
        playMusic();
    }

    /**
     * Loads all objects in a level based on levelNum.
     */
    public void initialLoad() {
        if (levelNum == 1) {
            addObject(new Grass(), 40, 395);
            addObject(new Grass(), 120, 395);
            addObject(new Grass(), 200, 395);
            addObject(new Grass(), 280, 395);
            addObject(new Grass(), 360, 395);
            addObject(new Grass(), 440, 395);
            addObject(new Grass(2), 520, 395);
            addObject(new Grass(1), 760, 365);
            addObject(new Grass(), 840, 365);
            addObject(new Grass(), 920, 365);
            addObject(new Grass(), 1000, 365);
            addObject(new Grass(), 1080, 365);
            addObject(new Grass(2), 1160, 365);
            addObject(new Cloud(1), 1320, 395);
            addObject(new Cloud(0), 1400, 395);
            addObject(new Cloud(0), 1480, 395);
            addObject(new BobRoss(), 120, 295);
            addObject(new EvilRabbit(), 360, 320);
            addObject(new EvilRabbit(), 280, 320);
            addObject(new EvilRabbit(), 920, 280);
            paletteX = 1450;
            paletteY = 290;
        }
        if (levelNum == 2) {
            addObject(new Cloud(), 40, 395);
            addObject(new Cloud(), 120, 395);
            addObject(new Cloud(2), 200, 395);
            addObject(new Cloud(1), 360, 290);
            addObject(new Cloud(2), 440, 290);
            addObject(new Cloud(1), 600, 395);
            addObject(new Cloud(2), 680, 395);
            addObject(new Cloud(1), 840, 290);
            addObject(new Cloud(2), 920, 290);
            addObject(new Cloud(1), 1080, 395);
            addObject(new Cloud(2), 1160, 395);
            addObject(new Cloud(1), 1320, 290);
            addObject(new Cloud(), 1400, 290);
            addObject(new Cloud(), 1480, 290);
            addObject(new BobRoss(), 120, 295);
            addObject(new EvilRabbit(), 400, 205);
            addObject(new EvilRabbit(), 640, 320);
            addObject(new EvilRabbit(), 860, 205);
            addObject(new EvilRabbit(), 1100, 320);
            paletteX = 1450;
            paletteY = 185;
        }
        if (levelNum == 3) {
            addObject(new Cloud(), 40, 155);
            addObject(new Cloud(2), 120, 155);
            addObject(new Cloud(1), 200, 235);
            addObject(new Cloud(), 280, 235);
            addObject(new Cloud(2), 360, 235);
            addObject(new Cloud(1), 440, 315);
            addObject(new Cloud(), 520, 315);
            addObject(new Cloud(2), 600, 315);
            addObject(new Grass(1), 680, 395);
            addObject(new Grass(), 720, 395);
            addObject(new Grass(2), 800, 395);
            addObject(new Cloud(1), 880, 315);
            addObject(new Cloud(), 960, 315);
            addObject(new Cloud(2), 1040, 315);
            addObject(new Cloud(1), 1120, 235);
            addObject(new Cloud(), 1200, 235);
            addObject(new Cloud(2), 1280, 235);
            addObject(new Cloud(1), 1360, 155);
            addObject(new Cloud(), 1440, 155);
            addObject(new Cloud(), 1520, 155);
            addObject(new BobRoss(), 100, 55);
            addObject(new EvilRabbit(), 520, 230);
            addObject(new EvilRabbit(), 1040, 230);
            addObject(new EvilRabbit(), 1440, 70);
            paletteX = 1450;
            paletteY = 50;
        }
        if (levelNum > 3) {
            victory = true;
            addObject(new Image("YouWin-Text.png"), 750, 200);
        }
    }

    /**
     * Plays "The Joy of Painting Theme" by Larry Owens in a loop.
     */
    public void playMusic() {
        long diffMillis = (long)((System.nanoTime() - musicStart)/1E6);
        //the song is 154861ms long
        if(diffMillis >= 154861) {
            musicStart =  System.nanoTime();
            Greenfoot.playSound("BackgroundLoop.mp3");
        }
    }

    /**
     * Checks if there is at least one enemy in the world.
     * 
     * @return true if there is at least one enemy, otherwise false.
     */
    public boolean isEnemy() {
        enemies = new ArrayList<>(getObjects(EvilRabbit.class));
        for(EvilRabbit x: enemies) {
            if(x != null) {
                return true;
            }
        }
        return false;
    }

    /**
     * @return whether or not the world is loaded.
     */
    public boolean isLoaded() {
        return loaded;
    }

    /**
     * @param loaded whether or not the world is loaded.
     */
    public void setLoaded(boolean loaded) {
        this.loaded = loaded;
    }

    /**
     * @return whether or not the palette has been added.
     */
    public boolean isPaletteAdded() {
        return paletteAdded;
    }

    /**
     * @param paletteAdded whether or not the palette has been added.
     */
    public void setPaletteAdded(boolean paletteAdded) {
        this.paletteAdded = paletteAdded;
    }

    /**
     * @return whether or not the player has won.
     */
    public boolean isVictory() {
        return victory;
    }

    /**
     * @param victory whether or not the player has won.
     */
    public void setVictory(boolean victory) {
        this.victory = victory;
    }

    /**
     * @return the time in milliseconds when the music started playing.
     */
    public static long getMusicStart() {
        return musicStart;
    }

    /**
     *
     * @param musicStart the time in milliseconds when the music started playing.
     */
    public static void setMusicStart(long musicStart) {
        Level.musicStart = musicStart;
    }

    /**
     * @return an ArrayList of the EvilRabbit objects in the world. 
     */
    public ArrayList<EvilRabbit> getEnemies() {
        return enemies;
    }

    /**
     * @param enemies an ArrayList of the EvilRabbit objects in the world. 
     */
    public void setEnemies(ArrayList<EvilRabbit> enemies) {
        this.enemies = enemies;
    }

    /**
     * @return the level the player is on.
     */
    public int getLevelNum() {
        return levelNum;
    }

    /**
     * @precondition must be >= 0
     * @param levelNum the level the player is on
     */
    public void setLevelNum(int levelNum) {
        this.levelNum = levelNum;
    }

    /**
     * @return the x coordinate that the palette is to be placed at.
     */
    public int getPaletteX() {
        return paletteX;
    }

    /**
     * @param paletteX the x coordinate that the palette is to be placed at.
     */
    public void setPaletteX(int paletteX) {
        this.paletteX = paletteX;
    }

    /**
     * @return the y coordinate that the palette is to be placed at.
     */
    public int getPaletteY() {
        return paletteY;
    }

    /**
     * @param paletteY the y coordinate that the palette is to be placed at.
     */
    public void setPaletteY(int paletteY) {
        this.paletteY = paletteY;
    }

    /**
     * @return a String representation of the object.
     */
    public String toString() {
        return "Level{" +
                "loade = " + loaded +
                ", paletteAdded = " + paletteAdded +
                ", victory = " + victory +
                ", enemies = " + enemies +
                ", levelNum = " + levelNum +
                ", paletteX = " + paletteX +
                ", paletteY = " + paletteY +
                '}';
    }
}
