/*
 * BOB ROSS THE VIDEO GAME
 * by: Spencer Horspool
 * 
 * Credit for rabbit sprites goes to Re-Logic
 * Credit fot Bob Ross sprite concept goes to PixelBloc on Tumblr
 * 
 * This is a simple platformer game in which the user controls Bob Ross as he tries
 * to turn corrupted rabbits good again with the joy of painting!
 * 
 * Controls: 
 * up: jump
 * left/right: walk
 * space: fling paint
 * 
 * This program makes extensive use of collision detection in order to run. If Bob 
 * comes into contact with an evil rabbit or the bottom of the world, he dies. If 
 * am evil rabbit comes into contact witha a paint projectile, it turns into a good
 * rabbit. If Bob collects the paint palette after turning all of the evil rabbits 
 * good, he continues onto the next level.
 */

import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * The main menu of the game. 
 * Features title text and flashing "click to start" text.
 * An instance is created before any new game is started.
 */
public class MainMenu extends World {

    private Image startText;
    private long displayStartTime; 
    private boolean displayingStart;
    private boolean clicked;
    private boolean soundPlayed;
    
    /**
     * Constructs the default main menu.
     */
    public MainMenu() {    
        super(1500, 400, 1);
        startText = new Image("ClickToStart-Text.png");
        displayStartTime = 0;
        clicked = false;
        displayingStart = false;
        soundPlayed = false;
        addObject(new Image("BobRoss-Text.png"), 750, 75);
        addObject(new Image("TheVideoGame-Text.png"), 750, 150);
    }
    
    /**
     * Displays flashing "Click to Start" text.
     * Starts the game when clicked.
     */
    public void act() {
        if(Greenfoot.mouseClicked(this)) {
            clicked = true;
        }
        if(clicked) {
            if(!soundPlayed) {
                Greenfoot.playSound("LetsGetCrazy.mp3");
                displayStartTime = System.currentTimeMillis();
                soundPlayed = true;
            }
            if(displayingStart){
                removeObject(startText);
                displayingStart = false;
            } else {
                addObject(startText, 750, 250);
                displayingStart = true;
            }
            if(System.currentTimeMillis() - displayStartTime >= 2691){
                Greenfoot.setWorld(new Level(1));
            }
        } else if(System.currentTimeMillis() - displayStartTime >= 500) {
            if(displayingStart){
                removeObject(startText);
                displayingStart = false;
            } else {
                addObject(startText, 750, 250);
                displayingStart = true;
            }
            displayStartTime = System.currentTimeMillis();
        }
    }
    
    /**
     * @return the "Click to Start" Image.
     */
    public Image getStart() {
        return startText;
    }
    
    /**
     * @param startText an Image for the flashing text.
     */
    public void setStart(Image startText) {
        this.startText = startText;
    }
    
    /**
     * @return the time in milliseconds when the flashing text appeared.
     */
    public long getDisplayStartTime() {
        return displayStartTime;
    }
    
    /**
     * @param displayStartTime the time in milliseconds when the flashing text is said to have appeared.
     */
    public void setDisplayStartTime(long displayStartTime) {
        this.displayStartTime = displayStartTime;
    }

    /**
     * @return a String representation of the object.
     */
    public String toString() {
        return "MainMenu{" +
                "startText = " + startText +
                ", displayStartTime = " + displayStartTime +
                ", displayingStart = " + displayingStart +
                ", clicked = " + clicked +
                ", soundPlayed = " + soundPlayed +
                '}';
    }
}