import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * A projectile launched by BobRoss that turns EvilRabbits into GoodRabbits.
 */
public class PaintSplat extends Actor {
    
    private boolean left;
    private int frame;
    private int speed;
    private static final String[] MOVE_L = {"PaintSplatL_0.png", "PaintSplatL_1.png", "PaintSplatL_2.png", "PaintSplatL_3.png"};
    private static final String[] MOVE_R = {"PaintSplatR_0.png", "PaintSplatR_1.png", "PaintSplatR_2.png", "PaintSplatR_3.png"};
    
    /**
     * The sole constructor for PainSplat objects.
     * @param left whether or not the PaintSplat is facing left.
     */
    public PaintSplat(boolean left) {
        this.left = left;
        this.speed = 30;
        if(left) {
            setImage("PaintSplatL_0.png");
        } else {
            setImage("PaintSplatR_0.png");
        }
    }
    /**
     * Moves the object at a fixed rate in a specifed direction.
     * Checks if the projectile has collided with an EvilRabbit.
     */
    public void act() {
        if(left) {
            moveL();
        } else {
            moveR();
        }
        checkCondition();
    }
    
    /**
     * Moves the object left and animates it. 
     */
    public void moveL() {
        if(frame != 3){
            setImage(MOVE_L[frame]);
        } else {
            setImage(MOVE_L[frame]);
            frame = 0;
            setLocation(getX() - speed, getY());
            return;
        }
        setLocation(getX() - speed, getY());
        frame++;
    }
    
    /**
     * Moves the object right and animates it. 
     */
    public void moveR() {
        if(frame != 3){
            setImage(MOVE_R[frame]);
        } else {
            setImage(MOVE_R[frame]);
            frame = 0;
            setLocation(getX() + speed, getY());
            return;
        }
        setLocation(getX() + speed, getY());
        frame++;
    }
    
    /**
     * Checks if the projectile has collided with an EvilRabbit. If so,
     * the EvilRabbit is replaced with a GoodRabbit.
     */
    public void checkCondition() {
        Actor enemy = getOneIntersectingObject(EvilRabbit.class);
        if(enemy != null) {
            Rabbit cleansedEnemy = (Rabbit)enemy;
            getWorld().addObject(new GoodRabbit(cleansedEnemy.isLeft()), cleansedEnemy.getX(), cleansedEnemy.getY());
            getWorld().removeObject(cleansedEnemy);
            getWorld().removeObject(this);
            return;
        }
        if(isAtEdge()) {
            getWorld().removeObject(this);
            return;
        }
    }

    /**
     * @return whether or not the projectile is facing left.
     */
    public boolean isLeft() {
        return left;
    }

    /**
     * @param whether or not the projectile is facing left.
     */
    public void setLeft(boolean left) {
        this.left = left;
    }

    /**
     * @return the animation frame the object is in.
     */
    public int getFrame() {
        return frame;
    }

    /**
     * @param frame the animation frame the object is in.
     */
    public void setFrame(int frame) {
        this.frame = frame;
    }

    /**
     * @return the horizontal speed of the object.
     */
    public int getSpeed() {
        return speed;
    }

    /**
     * @param speed the horizontal speed of the object.
     */
    public void setSpeed(int speed) {
        this.speed = speed;
    }

    /**
     * @return a String representation of the object.
     */
    public String toString() {
        return "PaintSplat{" +
                "left = " + left +
                ", frame = " + frame +
                ", speed = " + speed +
                '}';
    }
}
